# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from horizon import tabs

from django.utils.translation import ugettext_lazy as _


class DetailsTab(tabs.Tab):
    """Class to handle the data population of the test details views."""
    name = _("Details")
    slug = "details"
    template_name = \
        "project/connections/reachability_tests/_detail_overview.html"

    def get_context_data(self, request):
        reachabilitytest = self.tab_group.kwargs['reachabilitytest']
        return {"reachabilitytest": reachabilitytest}


class ReachabilityTestDetailTabs(tabs.TabGroup):
    slug = "reachability_test_details"
    tabs = (DetailsTab,)


class QuickDetailsTab(tabs.Tab):
    name = _("Quick Test Results")
    slug = "quick_details"
    template_name = \
        "project/connections/reachability_tests/_quick_detail_overview.html"

    def get_context_data(self, request):
        reachabilityquicktest = self.tab_group.kwargs['reachabilityquicktest']
        return {"reachabilityquicktest": reachabilityquicktest}


class QuickTestDetailTabs(tabs.TabGroup):
    slug = "quick_test_details"
    tabs = (QuickDetailsTab,)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Views for Connections tab.
"""

from horizon import exceptions
from horizon import tabs
from horizon.utils import memoized
from horizon_bsn.content.connections import tabs as project_tabs
from openstack_dashboard import api


class IndexView(tabs.TabbedTableView):
    tab_group_class = project_tabs.ConnectionsTabs
    template_name = 'project/connections/index.html'

    @memoized.memoized_method
    def _get_router(self):
        try:
            routers = api.neutron.router_list(
                self.request, **{'tenant_id': self.request.user.project_id})
            router = routers[0] if routers else None
            return router
        except Exception:
            msg = _('Unable to retrieve router for the current tenant.')
            exceptions.handle(self.request, msg)

    @memoized.memoized_method
    def _get_ports(self, router_id):
        try:
            ports = api.neutron.port_list(self.request,
                                          device_id=router_id)
        except Exception:
            ports = []
            msg = _('Unable to retrieve port details.')
            exceptions.handle(self.request, msg)
        return ports

    def get_context_data(self, **kwargs):
        context = super(IndexView, self).get_context_data(**kwargs)
        router = self._get_router()
        context["router"] = router
        return context

    def get_tabs(self, request, *args, **kwargs):
        router = self._get_router()
        ports = None
        if router:
            request.META['router_id'] = router.id
            ports = self._get_ports(router.id)
        return self.tab_group_class(request, router=router,
                                    ports=ports, **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import logging

from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from horizon import tabs

from horizon_bsn.api import neutron
from horizon_bsn.content.connections.network_template.tables \
    import NetworkTemplateAdminTable
from horizon_bsn.content.connections.network_template.tables \
    import NetworkTemplateTable
from horizon_bsn.content.connections.reachability_tests.tables \
    import ReachabilityTestsTable
from horizon_bsn.content.connections.routerrules import tabs as rr_tabs

import json

from openstack_dashboard.api import heat

LOG = logging.getLogger(__name__)


class NetworkTemplateAdminTab(tabs.TableTab):
    table_classes = (NetworkTemplateAdminTable,)
    name = _("Network Template Admin")
    slug = "network_template_tab_admin"
    template_name = "horizon/common/_detail_table.html"

    def allowed(self, request):
        return (self.request.path_info.startswith('/admin/') and
                super(NetworkTemplateAdminTab, self).allowed(request))

    def get_networktemplate_admin_data(self):
        try:
            networktemplates = neutron.networktemplate_list(self.request, **{})
            return networktemplates
        except Exception:
            return []


def get_stack_topology(request):
    try:
        assign = neutron.networktemplateassignment_get(
            request, request.user.tenant_id)
        networktemplate = neutron.networktemplate_get(
            request, assign.template_id)
    except Exception:
        return {"network_entities": "{}",
                "network_connections": "{}"}

    hc = heat.heatclient(request)
    # we only want the one that matches the network template
    stacks = [s for s in hc.stacks.list(tenant_id=request.user.tenant_id)
              if s.id == assign.stack_id]
    if not stacks:
        # leftover association, delete the assignment
        neutron.networktemplateassignment_delete(request,
                                                 request.user.tenant_id)
        return {"network_entities": "",
                "network_connections": ""}
    resources = hc.resources.list(assign.stack_id)
    entities = {}
    connections = []
    for res in resources:
        if res.resource_type in ('OS::Neutron::Router', 'OS::Neutron::Subnet'):
            entities[res.physical_resource_id] = {
                'properties': {'name': res.physical_resource_id}
            }
        if res.resource_type == 'OS::Neutron::RouterInterface':
            connections.append({
                'source': res.physical_resource_id.split(':subnet_id=')[0],
                'destination':
                    res.physical_resource_id.split('subnet_id=')[-1],
                'expected_connection': 'forward'
            })
    resp = {'template': networktemplate,
            'assign': assign,
            'network_entities': json.dumps(entities),
            'network_connections': json.dumps(connections),
            'stack_resources': resources,
            'stack': stacks[0]}
    return resp


def is_heat_available(request):
    try:
        heat.heatclient(request)
        return True
    except Exception:
        return False


class NetworkTemplateTab(tabs.TableTab):
    table_classes = (NetworkTemplateTable,)
    name = _("Network Template")
    slug = "network_template_tab"
    template_name = "horizon/common/_detail_table.html"

    def allowed(self, request):
        # don't show tab to tenants if heat isn't installed
        if not is_heat_available(request):
            return False
        # don't show the regular template tab to admins
        return (not request.path_info.startswith('/admin/')
                and super(NetworkTemplateTab, self).allowed(request))

    def get_networktemplate_data(self):
        try:
            topology = get_stack_topology(self.request)
            if not topology.get('assign'):
                return []
            tabledata = {
                'template_id': topology['template'].id,
                'template_name': topology['template'].name,
                'stack_id': topology['stack'].id,
                'heat_stack_name': topology['stack'].stack_name,
                'description': topology['stack'].description,
                'status': topology['stack'].status,
                'stack_status': topology['stack'].stack_status,
                'stack_status_reason': topology['stack'].stack_status_reason,
                'resources': mark_safe('<br>'.join([
                    ('%s (%s)' % (r.resource_name,
                                  r.resource_type)).replace(' ', '&nbsp;')
                    for r in topology['stack_resources']]))
            }
            return [tabledata]
        except Exception:
            return []


class ReachabilityTestsTab(tabs.TableTab):
    table_classes = (ReachabilityTestsTable,)
    name = _("Reachability Tests")
    slug = "reachabilitytest_tab"
    template_name = "horizon/common/_detail_table.html"

    def get_reachabilitytests_data(self):
        try:
            reachabilitytests = neutron.reachabilitytest_list(self.request,
                                                              **{})
            return reachabilitytests
        except Exception:
            return []


class ConnectionsTabs(tabs.TabGroup):
    slug = "connections_tabs"
    # TODO(kevinbenton): re-enabled top talkers once implemented
    # tabs = (NetworkTemplateTab, ReachabilityTestsTab, TopTalkersTab)
    sticky = True
    tabs = (ReachabilityTestsTab, NetworkTemplateTab, NetworkTemplateAdminTab,
            rr_tabs.RulesGridTab, rr_tabs.RouterRulesTab)



# Copyright 2013,  Big Switch Networks, Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy
from horizon import tables
from horizon_bsn.content.connections.routerrules import rulemanager
from openstack_dashboard import policy

LOG = logging.getLogger(__name__)


class AddRouterRule(policy.PolicyTargetMixin, tables.LinkAction):
    name = "create"
    verbose_name = _("Add Router Rule")
    url = "horizon:project:connections:routerrules:addrouterrule"
    classes = ("ajax-modal",)
    icon = "plus"
    policy_rules = (("network", "update_router"),)

    def get_link_url(self, datum=None):
        return reverse(self.url)


class RemoveRouterRule(policy.PolicyTargetMixin, tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Router Rule",
            u"Delete Router Rules",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Router Rule",
            u"Deleted Router Rules",
            count
        )

    failure_url = 'horizon:project:connections:index'
    policy_rules = (("network", "update_router"),)

    def delete(self, request, obj_id):
        router_id = self.table.kwargs['router'].id
        rulemanager.remove_rules(request, obj_id,
                                 router_id=router_id)


class RouterRulesTable(tables.DataTable):
    priority = tables.Column("priority", verbose_name=_("Priority"))
    source = tables.Column("source", verbose_name=_("Source CIDR"))
    destination = tables.Column("destination",
                                verbose_name=_("Destination CIDR"))
    action = tables.Column("action", verbose_name=_("Action"))
    nexthops = tables.Column("nexthops", verbose_name=_("Next Hops"))

    def get_object_display(self, rule):
        return "(%(action)s) %(source)s -> %(destination)s" % rule

    def get_object_id(self, datum):
        return datum.priority

    class Meta(object):
        name = "routerrules"
        verbose_name = _("Router Rules")
        table_actions = (AddRouterRule, RemoveRouterRule)
        row_actions = (RemoveRouterRule, )



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Views for managing instances.
"""
from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy

from horizon import forms

from horizon_bsn.content.connections.network_template import admin_forms
from horizon_bsn.content.connections.network_template \
    import forms as project_forms


class ApplyTemplateView(forms.ModalFormView):
    form_class = project_forms.ApplyTemplateForm
    template_name = 'project/connections/network_template/apply_template.html'
    success_url = reverse_lazy('horizon:project:connections:index')

    def get_context_data(self, **kwargs):
        context = super(ApplyTemplateView, self).get_context_data(**kwargs)
        context['template_id'] = self.request.path_info.split('/')[-1]
        return context


class PopulateTemplateView(forms.ModalFormView):
    form_class = project_forms.ApplyTemplateForm
    template_name = 'project/connections/network_template/temp_page.html'

    def get_context_data(self, **kwargs):
        context = super(PopulateTemplateView, self).get_context_data(**kwargs)
        context['template_id'] = self.request.path_info.split('/')[-1]
        return context


class SelectTemplateView(forms.ModalFormView):
    form_class = project_forms.SelectTemplateForm
    template_name = 'project/connections/network_template/select_template.html'
    success_url = (
        'horizon:project:connections:network_template:populate_template')

    def get_success_url(self):
        return reverse(
            self.success_url,
            kwargs={'template_id': self.request.POST['network_templates']})


class RemoveTemplateView(forms.ModalFormView):
    form_class = project_forms.RemoveTemplateForm
    template_name = 'project/connections/network_template/remove_template.html'
    success_url = reverse_lazy('horizon:project:connections:index')
    failure_url = reverse_lazy('horizon:project:connections:index')


class CreateView(forms.ModalFormView):
    form_class = admin_forms.CreateUpdateNetworkTemplate
    success_url = reverse_lazy('horizon:admin:connections:index')
    template_name = 'project/connections/network_template/admin_create.html'


class DetailView(forms.ModalFormView):
    form_class = admin_forms.DetailNetworkTemplate
    success_url = reverse_lazy('horizon:admin:connections:index')
    template_name = 'project/connections/network_template/admin_create.html'



# Copyright 2013,  Big Switch Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.core.exceptions import ValidationError  # noqa
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon_bsn.content.connections.routerrules import rulemanager

LOG = logging.getLogger(__name__)


class RuleCIDRField(forms.IPField):
    """Extends IPField to allow ('any','external') keywords and requires CIDR

    """
    def __init__(self, *args, **kwargs):
        kwargs['mask'] = True
        super(RuleCIDRField, self).__init__(*args, **kwargs)

    def validate(self, value):
        keywords = ['any', 'external']
        if value in keywords:
            self.ip = value
        else:
            if '/' not in value:
                raise ValidationError(_("Input must be in CIDR format"))
            super(RuleCIDRField, self).validate(value)


class AddRouterRule(forms.SelfHandlingForm):
    priority = forms.IntegerField(label=_("Priority"), initial=-1,
                                  min_value=-1, max_value=3000, required=False,
                                  widget=forms.HiddenInput())
    source = RuleCIDRField(label=_("Source CIDR"),
                           widget=forms.TextInput())
    destination = RuleCIDRField(label=_("Destination CIDR"),
                                widget=forms.TextInput())
    action = forms.ChoiceField(label=_("Action"))
    nexthops = forms.MultiIPField(label=_("Optional: Next Hop "
                                          "Addresses (comma delimited)"),
                                  widget=forms.TextInput(), required=False)
    router_id = forms.CharField(label=_("Router ID"),
                                widget=forms.TextInput(attrs={'readonly':
                                                              'readonly'}))
    failure_url = 'horizon:project:connections:index'

    def __init__(self, request, *args, **kwargs):
        super(AddRouterRule, self).__init__(request, *args, **kwargs)
        self.fields['action'].choices = [('permit', _('Permit')),
                                         ('deny', _('Deny'))]

    def handle(self, request, data, **kwargs):
        try:
            if 'rule_to_delete' in request.POST:
                rulemanager.remove_rules(request,
                                         request.POST['rule_to_delete'],
                                         router_id=data['router_id'])
        except Exception:
            exceptions.handle(request, _('Unable to delete router rule.'))
        try:
            if 'priority' not in data:
                data['priority'] = -1
            if 'nexthops' not in data:
                data['nexthops'] = ''
            if data['source'] == '0.0.0.0/0':
                data['source'] = 'any'
            if data['destination'] == '0.0.0.0/0':
                data['destination'] = 'any'
            rule = {'priority': data['priority'],
                    'action': data['action'],
                    'source': data['source'],
                    'destination': data['destination'],
                    'nexthops': data['nexthops'].split(',')}
            rulemanager.add_rule(request, router_id=data['router_id'],
                                 newrule=rule)
            msg = _('Router rule action performed successfully.')
            LOG.debug(msg)
            messages.success(request, msg)
            return True
        except Exception as e:
            msg = _('Failed to add router rule %s') % e
            LOG.info(msg)
            messages.error(request, msg)
            redirect = reverse(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.http import Http404
from django.utils.safestring import mark_safe
from django.utils.translation import pgettext_lazy
from django.utils.translation import ugettext_lazy as _
from horizon import messages
from horizon import tables
from horizon_bsn.api import neutron
from openstack_dashboard.api import heat

import logging

LOG = logging.getLogger(__name__)


class RemoveTemplateAction(tables.LinkAction):
    name = "remove"
    url = "horizon:project:connections:network_template:remove"
    classes = ("ajax-modal", "btn-danger")
    verbose_name = _("Remove Network Template Instance")

    def allowed(self, request, datum):
        try:
            assign = neutron\
                .networktemplateassignment_get(request,
                                               request.user.tenant_id)
            if assign:
                return True
        except Exception as e:
            LOG.debug(e)
            return False


class ApplyTemplateAction(tables.LinkAction):
    name = "apply"
    verbose_name = _("Apply Network Template")
    url = "horizon:project:connections:network_template:select"
    classes = ("ajax-modal", "btn-create")

    def allowed(self, request, datum):
        try:
            assign = neutron\
                .networktemplateassignment_get(request,
                                               request.user.tenant_id)
            if assign:
                return False
        except Exception:
            return True


class StacksUpdateRow(tables.Row):
    ajax = True

    def can_be_selected(self, datum):
        return datum.stack_status != 'DELETE_COMPLETE'

    def get_data(self, request, stack_id):
        assignment = None
        try:
            assignments = neutron.networktemplateassignment_list(
                request, **{'stack_id': stack_id})
            if not assignments:
                raise Exception('Network template associationg not found.')
            assignment = assignments[0]

            template = neutron.networktemplate_get(request,
                                                   assignment.template_id)

            stack = heat.stack_get(request, stack_id)
            resources = heat.resources_list(request, stack.stack_name)
            if stack.stack_status == 'DELETE_COMPLETE':
                # returning 404 to the ajax call removes the
                # row from the table on the ui
                raise Http404
            rowdata = {
                'template_id': assignment.template_id,
                'template_name': template.name,
                'stack_id': stack_id,
                'heat_stack_name': stack.stack_name,
                'description': stack.description,
                'status': stack.status,
                'stack_status': stack.stack_status,
                'stack_status_reason': stack.stack_status_reason,
                'resources': mark_safe('<br>'.join([
                    ('%s (%s)' % (r.resource_name,
                                  r.resource_type)).replace(' ', '&nbsp;')
                    for r in resources]))
            }
            return rowdata
        except Http404:
            try:
                # remove corresponding network template
                if assignment:
                    neutron.networktemplateassignment_delete(
                        request, assignment.id)
                msg = _('Removed template association for stack_id %s') % \
                    stack_id
                LOG.debug(msg)
                messages.success(request, msg)
            except Exception as e:
                msg = _('Network template association removal failed '
                        'due to %s') % e
                LOG.error(msg)
                messages.error(request, msg)
            raise
        except Exception as e:
            raise


class NetworkTemplateTable(tables.DataTable):
    STATUS_CHOICES = (
        ("Complete", True),
        ("Failed", False),
    )
    STACK_STATUS_DISPLAY_CHOICES = (
        ("init_in_progress", pgettext_lazy("current status of stack",
                                           u"Init In Progress")),
        ("init_complete", pgettext_lazy("current status of stack",
                                        u"Init Complete")),
        ("init_failed", pgettext_lazy("current status of stack",
                                      u"Init Failed")),
        ("create_in_progress", pgettext_lazy("current status of stack",
                                             u"Create In Progress")),
        ("create_complete", pgettext_lazy("current status of stack",
                                          u"Create Complete")),
        ("create_failed", pgettext_lazy("current status of stack",
                                        u"Create Failed")),
        ("delete_in_progress", pgettext_lazy("current status of stack",
                                             u"Delete In Progress")),
        ("delete_complete", pgettext_lazy("current status of stack",
                                          u"Delete Complete")),
        ("delete_failed", pgettext_lazy("current status of stack",
                                        u"Delete Failed")),
        ("update_in_progress", pgettext_lazy("current status of stack",
                                             u"Update In Progress")),
        ("update_complete", pgettext_lazy("current status of stack",
                                          u"Update Complete")),
        ("update_failed", pgettext_lazy("current status of stack",
                                        u"Update Failed")),
        ("rollback_in_progress", pgettext_lazy("current status of stack",
                                               u"Rollback In Progress")),
        ("rollback_complete", pgettext_lazy("current status of stack",
                                            u"Rollback Complete")),
        ("rollback_failed", pgettext_lazy("current status of stack",
                                          u"Rollback Failed")),
        ("suspend_in_progress", pgettext_lazy("current status of stack",
                                              u"Suspend In Progress")),
        ("suspend_complete", pgettext_lazy("current status of stack",
                                           u"Suspend Complete")),
        ("suspend_failed", pgettext_lazy("current status of stack",
                                         u"Suspend Failed")),
        ("resume_in_progress", pgettext_lazy("current status of stack",
                                             u"Resume In Progress")),
        ("resume_complete", pgettext_lazy("current status of stack",
                                          u"Resume Complete")),
        ("resume_failed", pgettext_lazy("current status of stack",
                                        u"Resume Failed")),
        ("adopt_in_progress", pgettext_lazy("current status of stack",
                                            u"Adopt In Progress")),
        ("adopt_complete", pgettext_lazy("current status of stack",
                                         u"Adopt Complete")),
        ("adopt_failed", pgettext_lazy("current status of stack",
                                       u"Adopt Failed")),
        ("snapshot_in_progress", pgettext_lazy("current status of stack",
                                               u"Snapshot In Progress")),
        ("snapshot_complete", pgettext_lazy("current status of stack",
                                            u"Snapshot Complete")),
        ("snapshot_failed", pgettext_lazy("current status of stack",
                                          u"Snapshot Failed")),
        ("check_in_progress", pgettext_lazy("current status of stack",
                                            u"Check In Progress")),
        ("check_complete", pgettext_lazy("current status of stack",
                                         u"Check Complete")),
        ("check_failed", pgettext_lazy("current status of stack",
                                       u"Check Failed")),
    )

    template_name = tables.Column("template_name",
                                  verbose_name=_("Template Name"))

    stack_id = tables.Column("stack_id", hidden=True)

    heat_stack_name = tables.Column("heat_stack_name",
                                    verbose_name=_("Heat Stack Name"))

    description = tables.Column("description", verbose_name=_("Description"))

    # status = tables.Column("status", verbose_name=_("Status"))

    resources = tables.Column("resources", verbose_name=_("Resources"))

    status = tables.Column("status",
                           hidden=True,
                           status=True,
                           status_choices=STATUS_CHOICES)

    stack_status = tables.Column("stack_status",
                                 verbose_name=_("Status"),
                                 display_choices=STACK_STATUS_DISPLAY_CHOICES)

    def get_object_id(self, template_stack):
        return template_stack['stack_id']

    class Meta(object):
        multi_select = False
        name = "networktemplate"
        verbose_name = _("Network Template")
        table_actions = (ApplyTemplateAction, RemoveTemplateAction)
        status_columns = ["status", ]
        row_class = StacksUpdateRow
        # row_actions = tuple()


class DeleteTemplateAction(tables.DeleteAction):
    data_type_singular = _("Network Template")
    data_type_plural = _("Network Templates")

    def delete(self, request, obj_id):
        try:
            neutron.networktemplate_delete(request, obj_id)
        except Exception as e:
            LOG.info(str(e))
            messages.error(
                request, _("Unable to delete template. Template may "
                           "be in use by a tenant."))


class CreateTemplateAction(tables.LinkAction):
    name = "create"
    verbose_name = _("Create Network Template")
    url = "horizon:admin:connections:network_template_admin:create"
    classes = ("ajax-modal", "btn-create")


class NetworkTemplateAdminTable(tables.DataTable):
    id = tables.Column("id", verbose_name=_("Template ID"), hidden=True)
    name = tables.Column(
        "name", verbose_name=_("Name"),
        link="horizon:admin:connections:network_template_admin:detail")

    class Meta(object):
        name = "networktemplate_admin"
        verbose_name = _("Network Template Administration")
        table_actions = (CreateTemplateAction, DeleteTemplateAction)
        row_actions = (DeleteTemplateAction,)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf.urls import include  # noqa
from django.conf.urls import patterns  # noqa
from django.conf.urls import url  # noqa
from horizon_bsn.content.connections.network_template \
    import admin_urls as network_template_urls
from horizon_bsn.content.connections import urls as purl
from horizon_bsn.content.connections_admin.views import IndexView
import os

LIB_PATH = os.path.join(os.path.abspath(os.path.dirname(purl.__file__)),
                        'templates', 'connections', 'lib')

JS_PATH = os.path.join(os.path.abspath(os.path.dirname(purl.__file__)),
                       'templates', 'connections', 'js')

CSS_PATH = os.path.join(os.path.abspath(os.path.dirname(purl.__file__)),
                        'templates', 'connections', 'css')

urlpatterns = patterns(
    '',
    url(r'^$', IndexView.as_view(), name='index'),
    url(r'network_template_admin/',
        include(network_template_urls, namespace='network_template_admin')),
    url(r'^lib/(?P<path>.*)$', 'django.views.static.serve',
        {'document_root': LIB_PATH, 'show_indexes': True}),
    url(r'^js/(?P<path>.*)$', 'django.views.static.serve',
        {'document_root': JS_PATH, 'show_indexes': True}),
    url(r'^css/(?P<path>.*)$', 'django.views.static.serve',
        {'document_root': CSS_PATH, 'show_indexes': True}),
)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf.urls import patterns  # noqa
from django.conf.urls import url  # noqa

from horizon_bsn.content.connections.network_template import views


urlpatterns = patterns(
    '',
    url(r'^apply_template/(?P<template_id>[^/]+)$',
        views.ApplyTemplateView.as_view(), name='apply'),
    url(r'^select_template/$',
        views.SelectTemplateView.as_view(), name='select'),
    url(r'^populate_template/(?P<template_id>[^/]+)$',
        views.PopulateTemplateView.as_view(), name='populate_template'),
    url(r'^remove_template/$',
        views.RemoveTemplateView.as_view(), name='remove'),
)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import httplib
import logging as log

session_map = {}
HASH_HEADER = 'Floodlight-Verify-Path'


def request(url, prefix="/api/v1/data/controller/", method='GET',
            data='', hashPath=None, host="127.0.0.1:8080", cookie=None):
    headers = {'Content-type': 'application/json'}

    if cookie:
        headers['Cookie'] = 'session_cookie=%s' % cookie

    if hashPath:
        headers[HASH_HEADER] = hashPath

    connection = httplib.HTTPSConnection(host)

    try:
        connection.request(method, prefix + url, data, headers)
        response = connection.getresponse()
        ret = (response.status, response.reason, response.read(),
               response.getheader(HASH_HEADER))
        if response.status >= 300:
            log.info('Controller REQUEST: %s %s:body=%r' %
                     (method, host + prefix + url, data))
            log.info('Controller RESPONSE: status=%d reason=%r, data=%r,'
                     'hash=%r' % ret)
        return ret
    except Exception as e:
        log.error("Controller REQUEST exception: %s" % e)
        raise


def get(cookie, url, server, port, hashPath=None):
    host = "%s:%d" % (server, port)
    return request(url, hashPath=hashPath, host=host, cookie=cookie)


def post(cookie, url, server, port, data, hashPath=None):
    host = "%s:%d" % (server, port)
    return request(url, method='POST', hashPath=hashPath, host=host, data=data,
                   cookie=cookie)


def patch(cookie, url, server, port, data, hashPath=None):
    host = "%s:%d" % (server, port)
    return request(url, method='PATCH', hashPath=hashPath, host=host,
                   data=data, cookie=cookie)


def put(cookie, url, server, port, data, hashPath=None):
    host = "%s:%d" % (server, port)
    return request(url, method='PUT', hashPath=hashPath, host=host, data=data,
                   cookie=cookie)


def delete(cookie, url, server, port, hashPath=None):
    host = "%s:%d" % (server, port)
    return request(url, method='DELETE', hashPath=hashPath, host=host,
                   cookie=cookie)



# Copyright 2013, Big Switch Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon.utils import memoized

from horizon_bsn.content.connections.routerrules import forms as rrforms

from openstack_dashboard import api


LOG = logging.getLogger(__name__)


class AddRouterRuleView(forms.ModalFormView):
    form_class = rrforms.AddRouterRule
    template_name = 'project/connections/routerrules/create.html'
    url = 'horizon:project:connections:index'
    page_title = _("Add Router Rule")

    def get_success_url(self):
        return reverse(self.url)

    @memoized.memoized_method
    def get_object(self):
        try:
            routers = api.neutron.router_list(
                self.request, **{'tenant_id': self.request.user.project_id})
            router = routers[0] if routers else None
            return router
        except Exception:
            redirect = reverse(self.url)
            msg = _("Unable to retrieve router.")
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(AddRouterRuleView, self).get_context_data(**kwargs)
        context['router'] = self.get_object()
        return context

    def get_initial(self, **kwargs):
        router = self.get_object()
        # store the router in the request so the rule manager doesn't have
        # to request it again from the API
        self.request.META['router'] = router
        return {"router_id": router.id,
                "router_name": router.name_or_id}
